/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package alignmentsystemexp;

import alignmentsystemexp.Creature;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Random;
import java.util.Set;

/**
 *
 * @author ckr
 */
public class Keeper {
    private List<Creature> creaturePool;
    private List<Creature> creatures;
    private Alignment alignment;
    private Allies allies;
    private double evolveRate;
    private StatisticsStrategy statisticsStrat;

    public void setEvolveRate(double evolveRate) {
        this.evolveRate = evolveRate;
    }

    public void setAlignment(Alignment alignment) {
        this.alignment = alignment;
    }

    public void setAllies(Allies allies) {
        this.allies = allies;
    }
    
    public double getEvolveRate() {
        return evolveRate;
    }
    private int maxCreatures;
    private Random rand;

    public List<Creature> getCreatures() {
        return creatures;
    }

    public List<Creature> getCreaturePool() {
        return creaturePool;
    }

    public Alignment getAlignment() {
        return alignment;
    }

    public Allies getAllies() {
        return allies;
    }

    public int getMaxCreatures() {
        return maxCreatures;
    }

    public Keeper(StatisticsStrategy alignStrat, List<Creature> creaturePool, List<Creature> creatures, Allies allies,
            Alignment alignment, double evolveRate, int maxCreatures, long seed) {
        this.creaturePool = creaturePool;
        this.creatures = creatures;
        this.allies = allies;
        this.alignment = alignment;
        this.evolveRate = evolveRate;
        this.maxCreatures = maxCreatures;
        this.rand = new Random(seed);
        this.statisticsStrat = alignStrat;
    }

    public void updateStatistics() {
        statisticsStrat.calculateStatistics(this, creatures);
        statisticsStrat.setStatistics(this, creatures);
    }

    public void waitForCreature() {
        double currentProb = 0.0;
        double prevProb = 0.0;
        double selected = rand.nextDouble();
        Creature chosen = null;
        for (Creature c : creaturePool) {
            prevProb = currentProb;
            currentProb += c.appearProb(this);
            if (prevProb <= selected && selected <= currentProb) {
                chosen = c;
            }
        }
        creatures.add(chosen);
    }

    public void evictCreatures(Alignment target, int numberToEvict) {
        for(int i=0; i<numberToEvict; i++){
            double currentDist = Double.POSITIVE_INFINITY;
            Creature evicted = null;
            for (Creature c : creatures) {
                List<Creature> tmp = new ArrayList<Creature>(creatures);
                tmp.remove(c);
                statisticsStrat.calculateStatistics(this, tmp);
                double dist = target.LpDistance(statisticsStrat.getAlignment(), 2.0);
                if (dist < currentDist) {
                    evicted = c;
                }
            }
            creatures.remove(evicted);
        }
    }

    public boolean evictNonWanted(Set<CreatureType> wanted){
        Set<CreatureType> unwanted = new HashSet<CreatureType>();
        for(Creature c : creatures){
            unwanted.add(c.getType());
        }
        unwanted.removeAll(wanted);
        Map<CreatureType, Integer> numOfType = new HashMap<CreatureType, Integer>();
        for(Creature c : creatures){
            if(numOfType.get(c.getType()) == null){
                numOfType.put(c.getType(), 1);
            }
            numOfType.put(c.getType(), numOfType.get(c.getType()) + 1);
        }
        Creature chosen = null;
        for(Creature c : creatures){
            if(unwanted.contains(c.getType()) || numOfType.get(c.getType()) > 1){
                chosen = c;
            }
        }
        boolean res = false;
        if(chosen != null){
            creatures.remove(chosen);
            res = true;
        }
        return res;
    }

    public String printCreatures() {
        String res = "";
        for (Creature c : creatures) {
            res += c.getType() + " ";
        }
        return res;
    }
}
